from sandbox.Core.IEnvironment import IEnvironment
from sandbox.Core.RegisterEnvironment import REGISTERENV


@REGISTERENV("echo", "echo idea content")
class Echo(IEnvironment):

    def __init__(self):
        self.template_ = """"""

    def template(self) -> str:
        return self.template_

    def execute(self, file_path) -> str:
        return f"batcat {file_path}"

    def format(self, file_path) -> str:
        return ""

    @staticmethod
    def is_file_env(file_path) -> bool:
        return False

    def extract_meta(self, file_path) -> dict:
        return {}

    def work_filename(self) -> str:
        return ""



from sandbox.Core.IEnvironment import IEnvironment
from sandbox.Core.RegisterEnvironment import REGISTERENV


@REGISTERENV("cpp", "default cpp environment")
class Cpp(IEnvironment):

    def __init__(self):
        self.template_ = """// sandbox_idea:
// sandbox_name:
// sandbox_description:
// sandbox_env: cpp

#include <iostream>

int main(void) {
    std::cout << "Hello World!" << std::endl;
    return 0;
}
"""

    def template(self) -> str:
        return self.template_

    def execute(self, file_path) -> str:
        return f"g++ {file_path} -o out && ./out && rm ./out"

    def format(self, file_path) -> str:
        return f"clang-format -i {file_path}"

    @staticmethod
    def is_file_env(file_path) -> bool:
        with open(file_path, "r") as file:
            for line in file:
                if line.strip() == "// sandbox_env: cpp".strip():
                    return True
        return False

    def extract_meta(self, file_path) -> dict:
        ret = dict(
            sandbox_idea="",
            sandbox_name="",
            sandbox_description="",
            sandbox_env="cpp")
        with open(file_path, "r") as file:
            for line in file:
                term = line.split(":")[0].replace("// ", "")
                if term in ret.keys():
                    ret[term] = line.split(":")[1].strip()
        return ret

    def work_filename(self) -> str:
        return "main.cpp"



import os
import argparse

from sandbox.Core.RegisterEnvironment import load_env_namespace
from sandbox.Core.RegisterPipeline import load_pipeline_namespace

parser = argparse.ArgumentParser(
    description="sandbox framework")

parser.add_argument(
    '-i', '--ideas',
    default=os.path.expandvars(f"{os.environ.get('SANDBOX_IDEAS', '').strip()}/"),
    help="ideas repo path")


def main():
    subparsers = parser.add_subparsers(dest='pipeline')
    subparsers.required = True
    pipelines = load_pipeline_namespace()
    for pip in pipelines:
        pipelines[pip]["type"].declare_args(subparsers)

    args = parser.parse_args()
    env = load_env_namespace()

    pip = pipelines[args.pipeline]["type"](env)
    pip.run(args)


if __name__ == "__main__":
    main()



import os

from sandbox.Core.ExecuteParentShell import write_on_parent_shell
from sandbox.Core.IPipeline import IPipeline
from sandbox.Core.RegisterPipeline import REGISTERPIPELINE
from sandbox.Core.FzfHandler import fzf_handler, sniff_dir


@REGISTERPIPELINE("open", "open idea on your text editor")
class Open(IPipeline):
    def __init__(self, env_namespace):
        self.env_namespace_ = env_namespace

    @staticmethod
    def declare_args(subparser):
        parser_upload = subparser.add_parser('open')

        parser_upload.add_argument(
            '-e', '--editor',
            default=os.environ.get('SANDBOX_EDITOR', 'vim'),
            help='editor to use')

        parser_upload.add_argument(
            '-p', '--path',
            default="",
            help='idea or directory path')

        parser_upload.add_argument(
            '-w', '--work_idea',
            action="store_true",
            help='open current work idea')

    def _open_file(self, path, args):
        write_on_parent_shell(f"{args.editor} {path}")
        return 0

    def run(self, args) -> int:
        if args.path == "":
            args.path = args.ideas

        if os.path.isfile(args.path):
            return self._open_file(args.path, args)
        if args.work_idea:
            return self._open_file(f"{args.path}/main*", args)
        else:
            return self._open_file(fzf_handler(sniff_dir(args.path)), args)



import os

from sandbox.Core.IPipeline import IPipeline
from sandbox.Core.RegisterPipeline import REGISTERPIPELINE


@REGISTERPIPELINE("save", "save work idea")
class Save(IPipeline):
    def __init__(self, env_namespace):
        self.env_namespace_ = env_namespace

    @staticmethod
    def declare_args(subparser):
        parser_upload = subparser.add_parser('save')

        parser_upload.add_argument(
            '-p', '--path',
            default="",
            help='work file')

    def _save_file(self, args, meta):
        os.system(f"mkdir -p {args.ideas}/{meta['sandbox_idea']}")
        idea_name = f'{args.ideas}/{meta["sandbox_idea"]}/{meta["sandbox_name"]}.{args.path.split(".")[-1]}'
        os.system(f"cp {args.path} {idea_name}")

    def _get_work_idea_path(self, path):
        for file in os.listdir(path):
            filename = os.fsdecode(file)
            if filename.startswith("main"):
                return f"{path}/{file}"
        return ""

    def run(self, args) -> int:
        if args.path == "":
            args.path = args.ideas
        if os.path.isdir(args.path):
            args.path = self._get_work_idea_path(args.path)
        if os.path.isfile(args.path):
            for env in self.env_namespace_:
                if self.env_namespace_[env]["type"].is_file_env(args.path):
                    env = self.env_namespace_[env]["type"]()
                    self._save_file(args, env.extract_meta(args.path))
                    return 1

        return 1



import os

ENV_NAMESPACE = dict()


def REGISTERPIPELINE(name: str, description: str):
    """Register implementation.

    Args:
        name (Type): type of registered entity.
        description (str): implementation description.
    """
    def wrapper(cls):
        ENV_NAMESPACE[name] = dict(type=(cls),
                                   description=description)
        return cls
    return wrapper


def load_pipeline_namespace() -> dict:
    module = None
    for module in os.listdir(os.path.dirname(
            f"{'/'.join(__file__.split('/')[:-1])}/../Pipelines/")):
        if module == '__init__.py' or module[-3:] != '.py':
            continue
        __import__(f"sandbox.Pipelines.{module[:-3]}", locals(), globals())
    del module
    return ENV_NAMESPACE



from sandbox.Core.IEnvironment import IEnvironment
from sandbox.Core.RegisterEnvironment import REGISTERENV


@REGISTERENV("bash", "default bash environment")
class Bash(IEnvironment):

    def __init__(self):
        self.template_ = """# sandbox_idea:
# sandbox_name:
# sandbox_description:
# sandbox_env: bash

echo "hello world!"
"""

    def template(self) -> str:
        return self.template_

    def execute(self, file_path) -> str:
        return f"bash {file_path}"

    def format(self, file_path) -> str:
        return f""

    @staticmethod
    def is_file_env(file_path) -> bool:
        with open(file_path, "r") as file:
            for line in file:
                if line.strip() == "# sandbox_env: bash".strip():
                    return True
        return False

    def extract_meta(self, file_path) -> dict:
        ret = dict(
            sandbox_idea="",
            sandbox_name="",
            sandbox_description="",
            sandbox_env="bash")
        with open(file_path, "r") as file:
            for line in file:
                term = line.split(":")[0].replace("# ", "")
                if term in ret.keys():
                    ret[term] = line.split(":")[1].strip()
        return ret

    def work_filename(self) -> str:
        return "main.sh"



import fcntl
import termios

BACKSPACE = '\x08'


def write_on_parent_shell(cmd: str, NB: int = 0):
    backspace = BACKSPACE * NB
    cmd = f"{backspace}{cmd}\n"
    for c in cmd:
        fcntl.ioctl(2, termios.TIOCSTI, c)



from sandbox.Core.IEnvironment import IEnvironment
from sandbox.Core.RegisterEnvironment import REGISTERENV


@REGISTERENV("markdown", "default markdown environment")
class Markdown(IEnvironment):

    def __init__(self):
        self.template_ = """[//]: # (sandbox_idea: )
[//]: # (sandbox_name: )
[//]: # (sandbox_description: )
[//]: # (sandbox_env: markdown)

# IDEA
"""

    def template(self) -> str:
        return self.template_

    def execute(self, file_path) -> str:
        return f"glow {file_path}"

    def format(self, file_path) -> str:
        return f"echo \"nothing\""

    @staticmethod
    def is_file_env(file_path) -> bool:
        with open(file_path, "r") as file:
            for line in file:
                if line.strip() == "[//]: # (sandbox_env: markdown)".strip():
                    return True
        return False

    def extract_meta(self, file_path) -> dict:
        ret = dict(
            sandbox_idea="",
            sandbox_name="",
            sandbox_description="",
            sandbox_env="markdown")
        with open(file_path, "r") as file:
            for line in file:
                term = line.split(":")[1].replace(" # (", "")
                if term in ret.keys():
                    ret[term] = line.split(":")[2].replace(")", "").strip()
        return ret

    def work_filename(self) -> str:
        return "main.md"
